
; This code builds and displays a Super Metroid-like automap for Metroid (NES)
; Copyright 2002-2004 Parasyte
; http://parasyte.panicus.org/
;
; Notes:
; Simply overwrite the data at file address 0x01CA45 (192 bytes, max)
; with this code. Then set the hook by patching 0x01CB68 to 0x35,0xCA


.org $CA35		;assemble our code to run at this (CPU) address
.mem 8			;setup accumulator and index regs for NES code
.index 8


;--constants
MapXStart	= $18
MapYStart	= $38
MapPal		= $01
MapFlashPal	= $00
RoomTile	= $6F
NoRoomTile	= $6E


;--pointers
MapPointer	= $00		;These are our temporary variables.
Counter		= $02
MapY		= $03
MapX		= $04

FrameCount	= $2D
Ypos		= $4F		;Samus' Y-Coordinate on Map.
Xpos		= $50		;Samus' X-Coordinate on Map.

CurSprite	= $5B		;Current sprite pointer. The game's sprite engine handles this.

SpriteBuf	= $0200		;Where all the sprites are!


;--routines
Adiv32		= $C2BE		;Power-of-two division.
Adiv16		= $C2BF
Adiv8		= $C2C0
Amul32		= $C2C4		;Power-of-two multiplication.
Amul16		= $C2C5
Amul8		= $C2C6




;--Entry Point
; Inputs:	None
; Outputs:	None
; Description:	Builds a map display using sprites, based on the currently loaded room number

	dec Ypos		;These will be incremented again before leaving the routine.
	dec Xpos

	lda Ypos		;Calculate map position address, too lazy to explain how it works!
	pha
	jsr Adiv8		;A >> 3 (Divide by 8)
	sta MapPointer+1
	pla
	jsr Amul32		;A << 5 (Multiply by 32)
	clc
	adc Xpos
	sta MapPointer
	lda MapPointer+1
	adc #$00
	ora #$70
	sta MapPointer+1


	lda #$03		;Automap height.
	sta Counter
	lda #MapYStart
	sta MapY
	
	ldx CurSprite

--	ldy #$00		;Begin drawing map to spr-ram page at $0200.
	lda #MapXStart
	sta MapX

-	lda (MapPointer),y	;Get room number.
	cmp #$FF
	beq +			;Branch if null room number. (#$FF)

	lda #RoomTile
	bne ++			;Branch always.
+	lda #NoRoomTile

++	sta SpriteBuf+1,x	;Sprite tile index.
	lda MapY
	sta SpriteBuf,x		;Sprite Y-coordinate.
	lda #MapPal
	sta SpriteBuf+2,x	;Sprite palette.
	lda MapX
	sta SpriteBuf+3,x	;Sprite X-coordinate.
	clc
	adc #$08
	sta MapX		;Update X-coordinte for next sprite.
	inx
	inx
	inx
	inx
	iny
	cpy #$03		;Automap width.
	bne -

	lda MapY		;Increment Y-coordinate for next line of sprites.
	clc
	adc #$08
	sta MapY
	dec Counter
	beq +			;Exit when complete.

	lda MapPointer		;Increment map pointer with new Y-Coordinate.
	clc
	adc #$20
	sta MapPointer
	lda MapPointer+1
	adc #$00
	sta MapPointer+1
	jmp --			;Drawing complete.


+	lda FrameCount		;Change palette of middle map sprite every 8 frames.
	and #$08
	bne +
	ldy CurSprite
	lda #MapFlashPal
	sta SpriteBuf+(4*4)+2,y
	
+	stx CurSprite		;Update current sprite position.
	inc Ypos		;See above ... I told you so!
	inc Xpos

	jmp $E0C1		;And we're outta here!
